from wowpy.constants import WSC_API_ENDPOINT, logger
from wowpy.utils import wowza_query

class TargetStream:

  custom_data = {}
  stream_target_base = WSC_API_ENDPOINT + 'stream_targets' 
  stream_target_type = stream_target_base + '/{stream_type}'
  stream_target_single = stream_target_type + '/{stream_target_id}'                     # target delete, get, update
  stream_target_properties = stream_target_base + '/{stream_target_id}' + '/properties' # target properties

  @classmethod
  def create_target(cls, data):
    endpoint = cls.stream_target_type.format(stream_type='custom')
    response = wowza_query(endpoint=endpoint, method='post', data=data)
    stream_target_id = response['stream_target_custom']['id']
    return stream_target_id

  # Update target: primary url, stream name
  @classmethod
  def update_target(cls, stream_target_id, target_data):
    endpoint = cls.stream_target_single.format(stream_type='custom', stream_target_id=stream_target_id)
    response = wowza_query(endpoint=endpoint, method='patch', data=target_data)
    stream_target_id = response['stream_target_custom']['id']
    return stream_target_id

  # Update stream target properties
  @classmethod
  def update_properties(cls, stream_target_id, properties_data):
    for property_data in properties_data:
      cls.update_property(stream_target_id=stream_target_id, data=property_data)

  @classmethod
  def update_property(cls, stream_target_id, data):
    endpoint = cls.stream_target_properties.format(stream_target_id=stream_target_id)
    properties_response = wowza_query(endpoint=endpoint, method='post', data=data)
    return properties_response['property']

  @classmethod
  def get_targets(cls, stream_type):
    # Get the list of custom stream targets
    endpoint = cls.stream_target_type.format(stream_type=stream_type)
    response = wowza_query(endpoint=endpoint, method='get')
    return response['stream_targets_'+stream_type]

  @classmethod
  def delete_targets(cls, stream_type, stream_target_ids):
    # Delete custom target streams
    for stream_target_id in stream_target_ids:
      cls.delete_target(stream_type=stream_type,
                        stream_target_id=stream_target_id)

  # if we change a Live stream name, baton is not going to be able to delete it
  # so the Live stream will exist when we try to delete the target, causing a wowza
  # exception 
  @classmethod
  def delete_target(cls, stream_type, stream_target_id):
    try:
      # Delete a stream target
      endpoint = cls.stream_target_single.format(stream_type=stream_type,
                                                  stream_target_id=stream_target_id)
      wowza_query(endpoint=endpoint, method='delete')
    except Exception as excp:
      logger.info('Target with id {stream_target_id} can not be deleted, please delete it manually, {excp}'.format(
        stream_target_id=stream_target_id,
        excp=excp)
      )

  @classmethod
  def get_target(cls, stream_type, stream_target_id):
    endpoint = cls.stream_target_single.format(stream_type=stream_type,
                                              stream_target_id=stream_target_id)
    response = wowza_query(endpoint=endpoint, method='get')
    stream_target = response['stream_target_custom']
    return stream_target
    
  @classmethod
  def get_target_properties(cls, stream_type, stream_target_id):
    endpoint = cls.stream_target_properties.format(stream_target_id=stream_target_id)
    response = wowza_query(endpoint=endpoint, method='get')
    return response['properties']


import os
import json
import logging
from logging import Logger
from pkg_resources import resource_string

# Load config
wowza_config = resource_string(__name__, 'config.json')
wowza_settings = json.loads(wowza_config)['properties']

# Settings
WSC_API_VERSION = wowza_settings['WSC_API_VERSION']
WSC_API_PATH = wowza_settings['WSC_API_PATH']
WSC_API_ENDPOINT = WSC_API_PATH + WSC_API_VERSION + '/'
WSC_TARGET_NAME = wowza_settings['WSC_TARGET_NAME']

# Logger setup
loglevel = wowza_settings['LOG_LEVEL']
logger = Logger('')
logger.setLevel(loglevel)
ch = logging.StreamHandler()
ch.setLevel(loglevel)
logger.addHandler(ch)


import requests
import functools
import json
import os
from requests.exceptions import HTTPError, ConnectionError, Timeout
from wowpy.constants import logger
from wowpy.exceptions import DeleteRecordingException

# utils

def safe_run(func):
  @functools.wraps(func)
  def func_wrapper(*args, **kwargs):
    try:
      return func(*args, **kwargs)
    except Exception as excp:
      logger.error(excp)
      return 'not created'
  return func_wrapper

# core

def make_query(endpoint, method, headers={}, data={}, auth=(), description='not description'):
  logger.info('Endpoint: {}, Method: {}, Data: {}, Description: {}'.format(endpoint, method, data, description))
  try:
    request_method = getattr(requests, method)
    response = request_method(endpoint, json=data, headers=headers, auth=auth)
    logger.debug('Response is: {}'.format(vars(response)))
    response.raise_for_status()
    logger.info('Endpoint {} succesfully reached'.format(endpoint))
  except HTTPError as err:
    logger.error(err.response.text)
    err_data = json.loads(err.response.text)
    message = err_data['meta']['message']
    status = err_data['meta']['status']
    if status == 410:
      raise DeleteRecordingException
    else:
      raise Exception(message)
  except ConnectionError as err:
    logger.error(err.response.text)
    raise Exception(str(err))
  except Timeout as err:
    logger.error(err.response.text)
    raise Exception(str(err))
  except Exception as err:
    logger.error(err)
    raise Exception(str(err))
  return response

# wowza core

def wowza_query(endpoint, method, data={}):
  wsc_access_key = os.environ.get('WSC_ACCESS_KEY')
  wsc_api_key = os.environ.get('WSC_API_KEY')
  headers = {
      'Content-Type': 'application/json',
      'cache-control': 'no-cache',
      'wsc-access-key': wsc_access_key,
      'wsc-api-key': wsc_api_key
  }
  response = make_query(endpoint=endpoint, method=method, headers=headers, data=data)

  if method == 'delete':
    response = response.text
  else:
    response = response.json()

  return response


"""
Defines all ps_ams exceptions
"""


class WowPyException(Exception):
    """
    Base exception class
    """

    def __init__(self, msg=None, cause=None):
        self.msg = msg or self.msg
        self.cause = cause
        super().__init__(self.msg)


class DeleteRecordingException(WowPyException):
    """
    Thrown when a recording resource has been deleted.
    """
    msg = 'The recording resource has been deleted.'




from wowpy.constants import WSC_API_ENDPOINT, logger
from wowpy.utils import wowza_query
  
class Transcoder:
  transcoder_base = WSC_API_ENDPOINT + 'transcoders/{transcoder_id}'             # transcoder update timeout
  transcoder_start = transcoder_base + '/start'                                  # transcoder start
  transcoder_stop = transcoder_base + '/stop'                                    # transcoder stop
  transcoder_reset = transcoder_base + '/reset'                                  # transcoder reset
  transcoder_outputs = transcoder_base + '/outputs'                              # transcoder read/write outputs
  transcoder_output_single = transcoder_outputs + '/{output_id}'                 # transcoder output update, delete
  transcoder_targets = transcoder_outputs + '/{output_id}/output_stream_targets' # transcoder associate
  transcoder_target_single = transcoder_targets + '/{stream_target_id}'          # transcoder delete target 

  @classmethod
  def get_transcoder_outputs(cls, transcoder_id):
    # Get outputs for a transcoder
    endpoint = cls.transcoder_outputs.format(
      transcoder_id=transcoder_id
    )
    response = wowza_query(endpoint=endpoint, method='get')
    return response['outputs']

  @classmethod
  def update_transcoder(cls, transcoder_id, data):
    # Update transcoder timeout
    endpoint = cls.transcoder_base.format(
      transcoder_id=transcoder_id
    )
    wowza_query(endpoint=endpoint, method='patch', data=data)

  @classmethod
  def delete_target(cls, transcoder_id, output_id, stream_target_id):
    # Delete stream target created by wowza
    endpoint = cls.transcoder_target_single.format(
      transcoder_id=transcoder_id, 
      output_id=output_id, 
      stream_target_id=stream_target_id
    )
    wowza_query(endpoint=endpoint, method='delete')

  @classmethod
  def create_transcoder_output(cls, transcoder_id, data):
    # Add an output to the transcoder
    endpoint = cls.transcoder_outputs.format(
      transcoder_id=transcoder_id
    )
    response = wowza_query(endpoint=endpoint, method='post', data=data)
    return response['output']['id']

  @classmethod
  def delete_transcoder_output(cls, transcoder_id, output_id):
    endpoint = cls.transcoder_output_single.format(transcoder_id=transcoder_id, output_id=output_id)
    wowza_query(endpoint=endpoint, method='delete')

  @classmethod
  def update_transcoder_output(cls, transcoder_id, output_id, data):
    endpoint = cls.transcoder_output_single.format(transcoder_id=transcoder_id, output_id=output_id)
    response = wowza_query(endpoint=endpoint, method='patch', data=data)
    return response['output']

  @classmethod
  def associate_target_stream(cls, transcoder_id, output_id, stream_target_id):
    # Associate a target with a live stream
    data = {
      'output_stream_target': {
        'stream_target_id': stream_target_id,
        'use_stream_target_backup_url': 'false'
      }
    }
  
    endpoint = cls.transcoder_targets.format(
      transcoder_id=transcoder_id, 
      output_id=output_id
    )
    wowza_query(endpoint=endpoint, method='post', data=data)

  @classmethod
  def get_transcoder(cls, transcoder_id):
    endpoint = cls.transcoder_base.format(
      transcoder_id=transcoder_id
    )
    response = wowza_query(endpoint=endpoint, method='get')
    transcoder = response['transcoder']
    return transcoder

  @classmethod
  def start_transcoder(cls, transcoder_id):
    endpoint = cls.transcoder_start.format(
      transcoder_id=transcoder_id
    )
    response = wowza_query(endpoint=endpoint, method='put')
    transcoder = response['transcoder']
    return transcoder

  @classmethod
  def stop_transcoder(cls, transcoder_id):
    endpoint = cls.transcoder_stop.format(
      transcoder_id=transcoder_id
    )
    response = wowza_query(endpoint=endpoint, method='put')
    transcoder = response['transcoder']
    return transcoder

  @classmethod
  def reset_transcoder(cls, transcoder_id):
    endpoint = cls.transcoder_reset.format(
      transcoder_id=transcoder_id
    )
    response = wowza_query(endpoint=endpoint, method='put')
    transcoder = response['transcoder']
    return transcoder



from wowpy.livestreams import LiveStream
from wowpy.transcoders import Transcoder
from wowpy.targets import TargetStream

def get_resource_info(id):
  resource_info = {
    'input': {
      'livestream': {}
    },
    'output': {
      'transcoder': {},
      'targets': []
    },
    'schedule': {}
  }

  live_stream = LiveStream.get_live_stream(id)
  resource_info['input']['livestream'] = live_stream
  transcoder_info = Transcoder.get_transcoder(id)
  resource_info['output']['transcoder'] = transcoder_info
  stream_targets = live_stream['stream_targets']
  for stream_target in stream_targets:
    stream_target_id = stream_target['id']
    stream_target_info = TargetStream.get_target(stream_type='custom', stream_target_id=stream_target_id)
    stream_target_properties = TargetStream.get_target_properties(stream_type='custom', stream_target_id=stream_target_id)
    resource_info['output']['targets'].append({'config': stream_target_info, 'properties': stream_target_properties})

#   schedule_info = Schedule.get_schedule(scheduler_id=scheduler_id)
#   resource_info['schedule'] = schedule_info

  return resource_info